Documentation
¶
Index ¶
- func Backend() *backend
- func Factory(ctx context.Context, conf *logical.BackendConfig) (logical.Backend, error)
- func FactoryType(backendType logical.BackendType) logical.Factory
- func New() (interface{}, error)
- func StrListContains(haystack []string, needle string) bool
- type Destination
- type Document
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func Backend ¶
func Backend() *backend
Backend returns a private embedded struct of framework.Backend.
func FactoryType ¶
func FactoryType(backendType logical.BackendType) logical.Factory
FactoryType is a wrapper func that allows the Factory func to specify the backend type for the mock backend plugin instance.
func New ¶
func New() (interface{}, error)
New returns a new backend as an interface. This func is only necessary for builtin backend plugins.
func StrListContains ¶
StrListContains looks for a string in a list of strings.
Types ¶
type Destination ¶
type Destination struct {
TargetURL string `json:"target_url"`
SendEntityID bool `json:"send_entity_id"`
Timeout time.Duration `json:"timeout"`
FollowRedirects bool `json:"follow_redirects"`
Parameters []string `json:"params"`
Metadata map[string]string `json:"metadata"`
TargetCA []byte `yaml:"target_ca"`
}
Destination contains all the operator specified configuration.
type Document ¶
type Document struct {
Nonce string `json:"nonce"`
Path string `json:"path"`
Timestamp int64 `json:"timestamp"`
RequestID string `json:"request_id"`
EntityID string `json:"entity_id,omitempty"`
Parameters map[string]string `json:"params,omitempty"`
Metadata map[string]string `json:"metadata,omitempty"`
}
Document is serialized to JSON, signed using JWS and then POSTed to the target server where the signature must be verified.
Source Files
¶
Click to show internal directories.
Click to hide internal directories.